using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEditor;

[Serializable]
public class SZM_MapData : ScriptableObject
{
    [SerializeField]
    private int mapMaxLength;

    [SerializeField]
    private float min_x;

    [SerializeField]
    private float min_y;

    [SerializeField]
    private float terrainLerpFactor = 3.5f;

    [SerializeField]
    private short[] terrainTexIndex = new short[SZM_TerrainMaker.maxSupportTerrainTextureCount];

    [SerializeField]
    private List<Vector2> terrainCenterList = new List<Vector2>();

    [SerializeField]
    private List<SZM_TerrainDataCell> valueList = new List<SZM_TerrainDataCell>();

    private Dictionary<Vector2, SZM_TerrainDataCell> _allData = new Dictionary<Vector2, SZM_TerrainDataCell>();
    public Dictionary<Vector2, SZM_TerrainDataCell> AllData { get { return _allData; } }

    public void SetTerrainLerpFactor(float value)
    {
        terrainLerpFactor = value;
    }

    public float GetTerrainLerpFactor()
    {
        return terrainLerpFactor;
    }

    public short[] GetTerrainTextureData()
    {
        return terrainTexIndex.Clone() as short[];
    }

    public void SetTerrainIndex(int arrayIndex, short targetIndex)
    {
        terrainTexIndex[arrayIndex] = targetIndex;
    }

    public int GetMapMaxLength()
    {
        return mapMaxLength;
    }

    public float GetMinX()
    {
        return min_x;
    }

    public float GetMinY()
    {
        return min_y;
    }

    public void DeserializeData()
    {
        _allData.Clear();
        for (int i = 0; i < terrainCenterList.Count; i++)
        {
            byte[] terrainData = valueList[i].terrainData.Clone() as byte[];
            byte[] walkablaeData = valueList[i].walkableData.Clone() as byte[];
            SZM_TerrainDataCell tempCell = new SZM_TerrainDataCell();
            tempCell.terrainData = terrainData;
            tempCell.walkableData = walkablaeData;
            _allData.Add(terrainCenterList[i], tempCell);
        }
    }

    public void SerializeData()
    {
        terrainCenterList.Clear();
        valueList.Clear();

        float min_x = float.MaxValue;
        float max_x = float.MinValue;
        float min_y = float.MaxValue;
        float max_y = float.MinValue;

        int cellDataCount = SZM_TerrainMaker.squareLength * SZM_TerrainMaker.squareLength;

        foreach (KeyValuePair<Vector2, SZM_TerrainDataCell> data in _allData)
        {
            terrainCenterList.Add(data.Key);
            SZM_TerrainDataCell tempCell = new SZM_TerrainDataCell();
            tempCell.terrainData = data.Value.terrainData.Clone() as byte[];

            tempCell.walkableData = new byte[cellDataCount];
            for (int i = 0; i < cellDataCount; i++)
                tempCell.SetWalkable(i, 1);

            valueList.Add(tempCell);

            if (data.Key.x < min_x) min_x = data.Key.x;
            if (data.Key.x > max_x) max_x = data.Key.x;
            if (data.Key.y < min_y) min_y = data.Key.y;
            if (data.Key.y < max_y) max_y = data.Key.y;
        }

        this.min_x = min_x;
        this.min_y = min_y;
        mapMaxLength = (int)(Mathf.Max(max_x - min_x, max_y - min_y) / SZM_TerrainMaker.squareLength) + 1;
    }
}

[Serializable]
public struct SZM_TerrainDataCell
{
    [SerializeField]
    public byte[] terrainData;

    [SerializeField]
    public byte[] walkableData;

    public void SetWalkable(int i, byte value)
    {
        walkableData[i] = value;
    }
}